/************************************************************
 *                                                          *
 * midplay.c                                                *
 *                                                          *
 *----------------------------------------------------------*
 * Routines to play sound files via SoundBlaster DSP.       *
 *----------------------------------------------------------*
 * Copyright (c) 1994,1996 Ken Graham, All Rights Reserved  *
 *                                                          *
 * Redistribution and use in source and binary forms,       *
 * without modification, are permitted provided that the    *
 * following conditions are met:                            *
 * 1. Redistributions of source code must retain the above  *
 *    copyright notice, this list of conditions and the     *
 *    following disclaimer.                                 *
 * 2. Redistributions in binary form must reproduce the     *
 *    above copyright notice, this list of conditions and   *
 *    the following disclaimer in the documentation and/or  *
 *    other materials provided with the distribution.       *
 *                                                          *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY  *
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,   *
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
 * OF SUCH DAMAGE.                                          *
 *                                                          *
 ************************************************************
 *
 * $Author:   kag  $
 * $Revision:   1.7  $
 * $Date:   13 May 1997 22:47:14  $
 *
 * History:
 *
 * $Log:   D:/snd32/midi2/archives/midplay.c_v  $
 * 
 *    Rev 1.7   13 May 1997 22:47:14   kag
 * fixed verbose display
 * 
 *    Rev 1.6   12 May 1997 18:53:16   kag
 *  fixed "lost note off" problem
 * 
 *    Rev 1.5   11 May 1997 20:20:08   kag
 *  drums working correctly
 * 
 *    Rev 1.4   09 May 1997 23:25:32   kag
 * intermediate checkin
 * 
 *    Rev 1.3   04 May 1997  0:53:58   kag
 * made changes to fixe tempo timing routines
 * 
 *    Rev 1.2   02 May 1997 18:50:46   kag
 *  release build for SND32 ver 2.0
 * 
 *    Rev 1.1   04 May 1996 12:58:24   kag
 * added /r option (repeat). removed calls to FMinit and FMcleanup.
 * 
 *    Rev 1.0   01 May 1996 20:40:10   kag
 * Initial revision.
 * 
 *==========================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <conio.h>
#include <ctype.h>
#include "portable.h"
#include "snd32.h"
#include "opl3.h"

#define PROGNAME    "midplay"

void ClearDisplay(void);
void SetCursor(int row, int col);

VOID main(INT argc, CHAR *argv[])
{
    INT     i;
    INT     verbose = 0;
    INT     sndcard;
    INT     volume;
    INT     repeat  = FALSE;
    INT     done    = 0;
    CHAR   *ptr;
    CHAR    ifilename[64];
    MIDI   *song;
    time_t  StartTime;
    time_t  EndTime;

    volume = 127;
    fprintf(stderr, "%s: Copyright (c) 1995,1996 Ken Graham. All Rights Reserved.\n", PROGNAME);
    if (argc < 2) 
    {
        fprintf(stderr,"%s: Syntax: %s file[.mid] [/v] [/r] [/l vol]\n", PROGNAME, PROGNAME);
        fprintf(stderr,"  Input file is a .MID file (file extension is optional if .MID)\n");
        fprintf(stderr,"  /v will set verbose mode on\n");
        fprintf(stderr,"  /r will repeat song in a loop\n");
        fprintf(stderr, " /l will set mixer output level to vol (0-127)\n");
        exit(-1);
    }

   /*
    * initialize stuff
    */
    ifilename[0] = '\0';

   /*
    * scan for option flags
    */
    for (i=1;i<argc;i++) 
    {
        if (argv[i][0] == '/' || argv[i][0] == '-') 
        {
            switch(toupper(argv[i][1])) 
            {
            case 'V':
                verbose = 1;
                break;
            case 'L':
                if (argv[i][2] != ' ')
                {
                    volume = atoi(&argv[i][2]);
                }
                else 
                {
                    int j = 2;
    
                    while (argv[i][j] && argv[i][j] == ' ')
                    {
                        j++;
                    }
                    if (argv[i][j])
                    {
                        volume = atoi(&argv[i][j]);
                    }
                }
                if (volume > 127)
                {
                    volume = 127;
                }
                break;
            case 'R':
                repeat = TRUE;
                break;
            default:
                fprintf(stderr, "%s: Unknown Parameter: '%s'\n", PROGNAME, argv[i]);
                exit(1);
            }
        }
        else 
        {
            strlwr(argv[i]);
            if (strlen((char *)ifilename) == 0) 
            {
                if ((ptr = strchr(argv[i], '.')) == NULL) 
                {
                    strcpy((char *)ifilename, argv[i]);
                    strcat((char *)ifilename, ".mid");
                }
                else 
                {
                    strcpy((char *)ifilename, argv[i]);
                    *ptr = '\0';
                }
            }
        }
    }
   /*
    * check for card and set internal variables
    * must call SndCardType before any SND32 routines
    */
    sndcard = SndCardType();
    if (verbose == TRUE)
    {
        PrintSB(stdout);
        printf("/nPress a key to continue\n");
        getch();
    }
   /*
    * load MIDI Song and fill out MIDI structure
    */
    song = MIDI_Load((char *)ifilename);
    if (song)
    {
        extern INT  percussiveMode;
        extern INT  sb_voices;
        extern volatile VOICE   voice[];
        int     last_voice[MAX_VOICES];
        int     last_drum[MAX_PERC];
        ULONG   old_tempo;
        int     old_perc;

        StartTime = time(NULL);
       /*
        * play song once unless repeat is on
        */
        MIDI_Play(song, 1);

        printf("\nPress a key to abort playing\n");
        printf("Playing  \n");
        ClearDisplay();
        SetCursor(0,0);
        printf("Song: %s\n", ifilename);

        SetCursor(2, 0);
        printf("Voice   Chan   Freq   Inst     Perc Inst\n");
        for (i=0;i<sb_voices;i++)
        {
            printf(" %03d:\n", i);
            fflush(stdout);
        }
        if (verbose)
        {
            printf("MIDI format        = %3d   ", song->format);
            printf("MIDI tracks        = %d\n", song->tracks);
            printf("MIDI time division = %3d   ", song->tdivision);
            printf("MIDI tempo         = %3d\n", (ULONG)((ULONG)(1000000 * 60 )/(ULONG)song->tempo) );
            old_tempo = (ULONG)song->tempo;
            SetCursor(22, 60);
            printf("PercMode = %s\n", (percussiveMode ? "ON " : "OFF"));
            old_perc  = percussiveMode;
        }

        while (!done)
        {
            while (MIDI_Playing()) /* check while playing for keypress */
            {
                int     key;

                if (kbhit())
                {
                   /*
                    * got a key so let's bail out
                    */
                    key = getch();
                    switch(key)
                    {
                    case '+':        /* increase volume  */
                        volume += 16;
                        if (volume > 127)
                        {
                            volume = 127;
                        }
                        SetMixerVolume(volume, FM_VOL);
                        break;
                    case '-':        /* decrease volume  */
                        volume -= 16;
                        if (volume < 0)
                        {
                            volume = 0;
                        }
                        SetMixerVolume(volume, FM_VOL);
                        break;

                    default:
                        MIDI_Stop();
                        repeat = FALSE ;
                        break;
                    }
                }
                for (i=0;i<sb_voices;i++)
                {
                    if ((voice[i].chan + voice[i].freq) * voice[i].on != last_voice[i])
                    {
                        if (voice[i].on)
                        {
                            SetCursor(3+i, 9);
                            printf("%02d    %03d    %03d",
                                voice[i].chan,
                                voice[i].freq,
                                voice[i].inst);
                            fflush(stdout);
                        }
                        else
                        {
                            SetCursor(3+i, 8);
                            printf("                  ");
                            fflush(stdout);
                        }
                        last_voice[i] = (voice[i].chan + voice[i].freq) * voice[i].on;
                    }
                        
                }
                if (percussiveMode)
                {
                    for (i=0;i<5;i++)
                    {
                        if (voice[MAX_VOICES+i].on != last_drum[i])
                        {
                            if (voice[MAX_VOICES+i].on != last_drum[i])
                            {
                                SetCursor(3+i, 33);
                                printf("ON  %d", voice[MAX_VOICES+i].inst);
                            }
                            else
                            {
                                SetCursor(3+i, 33);
                                printf("OFF    ");
                            }
                            last_drum[i] = voice[MAX_VOICES+i].on;
                            fflush(stdout);
                        }
                    }
                }
                if (verbose)
                {
                    if ((ULONG)song->tempo != old_tempo)
                    {
                        SetCursor(22, 48);
                        printf("%3d", (ULONG)((ULONG)(1000000 * 60 )/(ULONG)song->tempo) );
                        fflush(stdout);
                        old_tempo = (ULONG)song->tempo;
                    }
                    if (old_perc != percussiveMode)
                    {
                        SetCursor(22, 60);
                        printf("PercMode = %s\n", (percussiveMode ? "ON " : "OFF"));
                        old_perc = percussiveMode;
                    }
                }
            }
            if (repeat == FALSE)
            {
                done = 1;
            }
            else
            {
                MIDI_Play(song, 1);
            }
        }

       /*
        * Print out time song was playing and exit
        */
        EndTime = time(NULL);
        SetCursor(24, 0);
        printf("\n\nStart Time  : %s", ctime((const time_t *)&StartTime));
        printf("End Time    : %s", ctime((const time_t *)&EndTime));
        printf("Elapsed time:  %.1f seconds\n", difftime(EndTime, StartTime));

        if (song)
        {
            MIDI_Free(song);
        }
    }
    else
    {
        printf("Unable to open song file '%s'.\n", ifilename);
    }

    MIDI_Stop();
    exit(0);
}

#include <dos.h>

void ClearDisplay()
{
    union REGS  regs;

    regs.w.cx = 0;
    regs.w.dx = 0x1850;
    regs.h.bh = 7;
    regs.w.ax = 0x0600;
    int386(0x10, (union REGS *)&regs, (union REGS *)&regs);
}

void SetCursor(int row, int col)
{
    char        pageno;
    union REGS  regs;

    regs.w.ax = 0x0f00;     // get video mode
    int386(0x10, (union REGS *)&regs, (union REGS *)&regs);
    pageno = regs.h.bh;     // get page number

    regs.w.ax = 0x200;      // set cursor posn
    regs.h.bh = pageno;     // set page #
    regs.h.dh = (char)row;  // set row
    regs.h.dl = (char)col;  // set col
    int386(0x10, (union REGS *)&regs, (union REGS *)&regs);
}
