/************************************************************
 *                                                          *
 * playsnds.c                                               *
 *                                                          *
 *----------------------------------------------------------*
 * Routines to play multiple sound files via SoundBlaster   *
 * DSP.                                                     *
 *----------------------------------------------------------*
 * Copyright (c) 1994,1996 Ken Graham, All Rights Reserved  *
 *                                                          *
 * Redistribution and use in source and binary forms,       *
 * without modification, are permitted provided that the    *
 * following conditions are met:                            *
 * 1. Redistributions of source code must retain the above  *
 *    copyright notice, this list of conditions and the     *
 *    following disclaimer.                                 *
 * 2. Redistributions in binary form must reproduce the     *
 *    above copyright notice, this list of conditions and   *
 *    the following disclaimer in the documentation and/or  *
 *    other materials provided with the distribution.       *
 *                                                          *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY  *
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,   *
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
 * OF SUCH DAMAGE.                                          *
 *                                                          *
 ************************************************************
 *
 * $Author
 * $Revision:   1.2  $
 * $Date:   02 May 1997 19:53:20  $
 *
 * History:
 *
 * $Log:   D:/snd32/midi2/archives/playsnds.c_v  $
 * 
 *    Rev 1.2   02 May 1997 19:53:20   kag
 *  cleaned up loose ends regarding exposing SB parameters
 * 
 *    Rev 1.1   02 May 1997 18:51:10   kag
 *  release build for SND32 ver 2.0
 * 
 *    Rev 1.0   06 May 1996 23:58:22   kag
 * Initial revision.
 * 
 ************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <conio.h>
#include <ctype.h>
#include "portable.h"
#include "snd32.h"

#define PROGNAME    "playsnds"

char spinchar[] = { "|\\-/" };

void main(int argc, char *argv[])
{
    INT          i;
    INT          chan[4];
    INT          cardtype;
    INT          count   = 0;
    INT          done    = 0;
    INT          verbose = FALSE;
    time_t       StartTime;
    time_t       EndTime;
    DIGSOUND     ds[4];

    fprintf(stderr, "%s: Copyright (c) 1994,1996 Ken Graham. All Rights Reserved.\n", PROGNAME);
    fprintf(stderr, "SND32 Routines %s\n", CopyRightStr);
    if (argc < 2) 
    {
        fprintf(stderr,"%s: Usage: playsnd sndfile.[WAV | RAW] [...]\n",
            PROGNAME);
        exit(1);
    }
   /*
    * Test for sound card type
    */
    cardtype = SndCardType();
    if (cardtype == NOCARD) 
    {
        fprintf(stderr, "%s: Error: sound card not found.\n", PROGNAME);
        exit(1);
    }
   /*
    * initialize ds structures
    */
    for (i=0;i<4;i++)
    {
        ds[i].name = NULL;;
        ds[i].data = NULL;
        ds[i].len  = 0;
    }

    for (i=1;i<argc;i++)
    {
        if (argv[i][0] == '-' || argv[i][0] == '/')
        {
            switch (toupper(argv[i][1]))
            {
            case 'V':
                verbose = TRUE;
                break;
            default:
                printf("%s: Error: unknown parameter '%s' ignored\n", 
                    PROGNAME, argv[i]);
                break;
            }
        }
        else
        {
            if (count < 4)
            {
                ds[count].name = argv[i];
                if (DSPload((DIGSOUND *)&ds[count]) == NULL) 
                {
                    fprintf(stderr, "%s: Error: Unable to load sound file '%s'\n", 
                        PROGNAME, argv[i]);
                    exit(1);
                }
                count++;
            }
            else
            {
                printf("%s: Error: maximum number of sounds exceeded\n",
                    PROGNAME);
            }
        }
    }
    if (verbose == TRUE)
    {
        PrintSB(stdout);
    }

    if (count > 0)
    {
        printf("Press a key to start sound(s)\n");
        getch();
        StartTime = time(NULL);
        for (i=0;i<count;i++)
        {
            chan[i] = DSPplay((DIGSOUND *)&ds[i], -1);
            if (verbose)
            {
                printf("sound %d: rate = %d\n", i, ds[i].rate);
            }
        }
    }
   /*
    * Play sound in a loop - until user presses ESC
    */
    printf("Sound should be playing. Press ESC to exit.\n");
    printf("  Playing");
    done = 0;
    while (!done) 
    {
        /* 
         * need to eat chars - DOS beep screws up irqs
         */
        if (kbhit()) 
        {
            INT key;
            /*
             * check for ESC
             */
            key = getch();
            switch (toupper(key))
            {
            case 0x1b:
                for (i=0;i<count;i++)
                {
                    DSPstop(-1);
                }
                done = 1;
                break;
            case '1':
                if (ds[0].len > 0)
                {
                    DSPplay((DIGSOUND *)&ds[0], -1);
                }
                break;
            case '2':
                if (ds[1].len > 0)
                {
                    DSPplay((DIGSOUND *)&ds[1], -1);
                }
                break;
            case '3':
                if (ds[2].len > 0)
                {
                    DSPplay((DIGSOUND *)&ds[2], -1);
                }
                break;
            case '4':
                if (ds[3].len > 0)
                {
                    DSPplay((DIGSOUND *)&ds[3], -1);
                }
                break;
            default:
                break;
            }
        }
        printf("\r%c", spinchar[++count&3]);
        fflush(stdout);
    }

    DSPcleanup();

    EndTime = time(NULL);
    printf("\n\nStart Time   : %s", ctime((time_t *)&StartTime));
    printf("End Time     : %s", ctime((time_t*)&EndTime));
    printf("Elapsed time :  %f seconds\n", difftime(EndTime, StartTime));
    
    exit(0);
}










