/************************************************************
 *                                                          *
 * portable.h                                               *
 *                                                          *
 *----------------------------------------------------------*
 * typedefs, defines and other things which make porting    *
 * from 16 bit to 32 bit much easier.                       *
 *----------------------------------------------------------*
 * Copyright (c) 1994 Ken Graham                            *
 *                                                          *
 * Redistribution and use in source and binary forms, with  *
 * or without modification, are permitted provided that the *
 * following conditions are met:                            *
 * 1. Redistributions of source code must retain the above  *
 *    copyright notice, this list of conditions and the     *
 *    following disclaimer.                                 *
 * 2. Redistributions in binary form must reproduce the     *
 *    above copyright notice, this list of conditions and   *
 *    the following disclaimer in the documentation and/or  *
 *    other materials provided with the distribution.       *
 *                                                          *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY  *
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,   *
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
 * OF SUCH DAMAGE.                                          *
 *                                                          *
 ************************************************************
 *
 * $Author:   kag  $
 * $Revision:   1.1  $
 * $Date:   02 May 1997 18:37:56  $
 *
 * History:
 *
 * $Log:   D:/snd32/midi2/archives/portable.h_v  $
   
      Rev 1.1   02 May 1997 18:37:56   kag
   No change.
   
      Rev 1.0   01 May 1996 20:40:06   kag
   Initial revision.
   
      Rev 1.0   15 Nov 1994 20:56:36   kag
   Initial revision.
 *
 ************************************************************/

#if !defined(_PORTABLE_H_)
#define _PORTABLE_H_ 1

typedef void            VOID;
typedef unsigned char   BYTE;       /* unsigned 8 bits      */
typedef unsigned short  WORD;       /* unsigned 16 bits     */
typedef unsigned int    UINT;       /* unsigned 16/32 bits  */
typedef unsigned long   DWORD;      /* unsigned 32 bits     */
typedef char            CHAR;       /* signed 8 bits        */
typedef short           SHORT;      /* signed 16 bits       */
typedef int             INT;        /* signed 16/32 bits    */
typedef long            LONG;       /* signed 32 bits       */

#define ULONG   DWORD               /* size: 32 bits            */
#define USHORT  WORD                /* size: 16 bits            */
#define UCHAR   BYTE                /* size:  8 bits            */

#if !defined(TRUE)
#define TRUE 1
#endif

#if !defined(FALSE)
#define FALSE 0
#endif

/*
 * Some compilers use memory.h, some use mem.h - go figure
 */
#if defined(USES_MEMORY_H)
#include <memory.h>
#else
#include <mem.h>
#endif

#if defined(USES_ASM_INTERRUPTS)
/*
 * Define interrupt disable & enable macros
 * These assembler directives are specific to GNU C 
 */
#define DISABLE() __asm__ ("cli");
#define ENABLE()  __asm__ ("sti");
#else
#define ENABLE()    _enable()
#define DISABLE()   _disable()
#endif

/*
 * defines for 32 bit FP_SEG & FP_OFF
 * Segment must be 1 byte for SB DAC routines
 */
#define D32FP_SEG(x) ((((DWORD)(x)) >> 16) & 0xFF)
#define D32FP_OFF(x) (((DWORD)(x)) & 0xFFFF)

#endif
